#include "lpc2106_init.cpp"
#include "lpc2106_memorymap.hpp"

#define FLASH_SECTOR 8

unsigned long buf[1024];
unsigned char* buffer;
short buffer_pointer=0;
char what = 1 ;

typedef void (*IAP) (unsigned long[], unsigned long[]);


//Clear buffer in RAM
void clear_buffer()
{
	for(short i = 0;i < 4096;i++)
	{
		buffer[i] = 0xFF;
		buffer_pointer = 0;
	}
	REG8(UART0_THR) = 'K';
	what = 0;
}

//Dump buffer from RAM to UART
void dump_buffer()
{
	for(short i = 0;((i < 4096) && (buffer[i]!=0xFF));i++)
	{
		while(!(REG8(UART0_LSR) & 0x20)){}
		REG8(UART0_THR) = buffer[i];
	}
	REG8(UART0_THR) = 'K';
	what = 0;
}

//Run code uploaded to RAM
void run_buffer()
{
	void (*temp)();
	long l =(long)buffer;
	temp = (void(*)())l;	//Cast the buffer pointer to function pointer
	temp();
	REG8(UART0_THR) = 'K';
	what = 0;
}

//Copy buffer RAM to the first part of the Flash sector given
void copy_to_flash(long sector)
{
	unsigned long command[5];	//Command pointer for IAP
	unsigned long result[2];	//Result pointer for IAP
	unsigned long vicmask = REG32(VICIntEnable);	//to save IT mask
	IAP iap_entry;
	REG32(VICIntEnClr) = 0xFFFFFFFF;	//Disable IT
	iap_entry=(IAP) IAP_LOCATION;	//Get IAP routine pointer
	command[0]=50;	//Prepare sector to write/erase
	command[1]=sector;	// from sector
	command[2]=sector;	// till sector
	iap_entry(command,result);	//launch IAP

	command[0]=51;	//Write sector
	command[1]=(long)(sector*0x2000);	//start of the sector: sector value*8192
	command[2]=(long)buf;	//Buffer start
	command[3]=4096;	//Buffer size
	command[4]=60000;	//Processor clock frequenci in kHz (External cristal:10 MHz M=6)
	iap_entry(command,result); //launch IAP
	REG32(VICIntEnable) = vicmask; //Enable IT
	REG8(UART0_THR) = result[0]+'0';
	what = 0;
}

//Dump flash memory from address
void dump_flash(long sector)
{
	char *temp=(char *)(sector*0x2000);
	for(short i = 0;(i < 4096) && (temp[i]!=0xFF);i++)
		{
			while(!(REG8(UART0_LSR) & 0x20)){}
			REG8(UART0_THR) = temp[i];
		}
	REG8(UART0_THR) = 'K';
	what = 0;
}

//Run code saved to Flash
void run_flash(long sector)
{
	void (*temp)();
	long l = (long)(sector*0x2000);
	temp = (void(*)())l;	//Cast sectro start pointer to function pointer
	temp();
	REG8(UART0_THR) = 'K';
	what = 0;
}



//Clear order from the end of the buffer
void end_order()
{
	buffer_pointer -= 5;
	buffer[buffer_pointer] = 0xFF;
	buffer[buffer_pointer+1] = 0xFF;
	buffer[buffer_pointer+2] = 0xFF;
	buffer[buffer_pointer+3] = 0xFF;
	buffer[buffer_pointer+4] = 0xFF;
}

// Chack if there is an order
char order(void)
{
	if (buffer_pointer >= 5)
	{	//Must start and end with '*'
		if ((buffer[buffer_pointer-5]=='*') && (buffer[buffer_pointer-1]=='*'))
		{
			if ((buffer[buffer_pointer-4]=='c') && //*clr* - Clear buffer in RAM
				(buffer[buffer_pointer-3]=='l') &&
				(buffer[buffer_pointer-2]=='r'))
				{end_order();return 1;}
			if ((buffer[buffer_pointer-4]=='d') &&//*dmp* - Dump buffer in RAM
				(buffer[buffer_pointer-3]=='m') &&
				(buffer[buffer_pointer-2]=='p'))
				{end_order();return 2;}
			if ((buffer[buffer_pointer-4]=='r') &&//*run* - Run code in RAM buffer
				(buffer[buffer_pointer-3]=='u') &&
				(buffer[buffer_pointer-2]=='n'))
				{end_order();return 3;}
			if ((buffer[buffer_pointer-4]=='c') &&//*ctf* - Copy RAM buffer to flash
				(buffer[buffer_pointer-3]=='t') &&
				(buffer[buffer_pointer-2]=='f'))
				{end_order();return 4;}
			if ((buffer[buffer_pointer-4]=='d') &&//*dfm* - Dump flash
				(buffer[buffer_pointer-3]=='f') &&
				(buffer[buffer_pointer-2]=='m'))
				{end_order();return 5;}
			if ((buffer[buffer_pointer-4]=='r') &&//*dfm* - Dump flash
				(buffer[buffer_pointer-3]=='f') &&
				(buffer[buffer_pointer-2]=='f'))
				{end_order();return 6;}

			return 0;

		}
		else {return 0;}
	}
	else {return 0;}
}

// Timer interrupt - disabeled
void timer0_irq_handler(void)
{
	REG8(UART0_THR) = 't';
	REG8(TIMER0_IR) = 0xFF;		// Clear the interrupt flag
}

//Default interrupt handler
void default_irq_handler(void)
{
	REG8(UART0_THR) = '@';
}

//UART interrupt handler
void uart0_irq_handler(void)
{
	switch (REG8(UART0_IIR) & 0x0E) //Mask with 1110
	{
		case (0x02) :				//In case of THRE interrupt (more often interrupt)
		{
			REG8(UART0_IER) = 0x01; //If we have nothing to write out
			break;
		}
		case (0x04) :				//In case od RDA interrupt (less often interrupt)
		{
			buffer[buffer_pointer++] = REG8(UART0_RBR);
			what=order();	//Check if there was an order or not
			if (!(REG8(UART0_IER) & 0x02)){REG8(UART0_IER) = 0x03;}
			break;
		}
	}
}

int main (void)
{
	lpc2106_startup::run();	//Init chip
	REG8(UART0_THR) = 'G'; //Write 'G' for Go
	buffer=(unsigned char*)buf; //Get char buffer from long buffer
	REG8(TIMER0_TCR) = 0x0; //Disable Timer interrupt
	lpc2106_startup::vectored_interrupts_init((unsigned long)timer0_irq_handler,
										(unsigned long) uart0_irq_handler,
										(unsigned long) default_irq_handler); //Set interrupt handlers
	if (!(REG8(UART0_IER) & 0x02)){REG8(UART0_IER) = 0x03;}
	while(1)	//Loop for order test
	{
		switch (what)
		{
			case (0) :  break;
			case (1) :  clear_buffer();break;
			case (2) :  dump_buffer();break;
			case (3) :  run_buffer();break;
			case (4) : 	copy_to_flash(FLASH_SECTOR);break;
			case (5) :  dump_flash(FLASH_SECTOR);break;
			case (6) :  run_flash(FLASH_SECTOR);break;
		}
	}
}
